from tenacity import retry, stop_after_attempt, wait_fixed

from aiddit.api.xhs_api import _get_xhs_account_info, _get_xhs_account_note_list, _get_note_detail_by_id
import aiddit.api.topic.prompt as prompt
import aiddit.model.google_genai as google_genai
import aiddit.utils as utils
from aiddit.model.google_genai import GenaiConversationMessage, GenaiMessagePart, MessageType
import os
import aiddit.api.history_note as history_note


def reference_note_available(xhs_user_id: str, reference_note_id: str):
    account_info = _get_xhs_account_info(xhs_user_id)
    account_history_note_path = _get_xhs_account_note_list(xhs_user_id)

    model = google_genai.MODEL_GEMINI_2_5_FLASH

    reference_note = _get_note_detail_by_id(reference_note_id)

    history_notes = utils.load_from_json_dir(account_history_note_path)

    history_messages = []

    history_messages.extend(history_note.build_history_note_messages(history_notes))

    # 参考帖子
    if reference_note.get("content_type") == "video" and reference_note.get("video", {}).get("video_url") is not None:
        reference_note_medias = [reference_note.get("video", {}).get("video_url")]
    else:
        reference_note_medias = [utils.oss_resize_image(i) for i in
                                 utils.remove_duplicates(reference_note.get("images"))]
    reference_note_prompt = prompt.REFERENCE_NOTE_PROVIDER_PROMPT.format(
        title=reference_note.get("title"),
        body_text=reference_note.get("body_text"),
        image_count=len(reference_note_medias)
    )
    reference_note_conversation_user_message = google_genai.GenaiConversationMessage.text_and_images(
        reference_note_prompt, reference_note_medias)
    history_messages.append(reference_note_conversation_user_message)

    # 获取当前文件的目录
    current_file_dir = os.path.dirname(os.path.abspath(__file__))
    print("current_file_dir ---", current_file_dir)
    # 专家知识
    expert_knowledge_path = os.path.join(current_file_dir, "../expert/topic_theory_expert.txt")
    print("expert_knowledge_path ---", expert_knowledge_path)

    if os.path.exists(expert_knowledge_path):
        expert_knowledge_conversation_user_message = google_genai.GenaiConversationMessage.text_and_images(
            f"小红书内容创作的理论基石：基于特定账号人设与参考帖子的选题策略。从中你可以学习 了解小红书内容创作的理论基石，基于特定账号人设与参考帖子的选题策略。",
            expert_knowledge_path)
        history_messages.append(expert_knowledge_conversation_user_message)
    else:
        print("expert_knowledge_path not exist , skip expert knowledge")

    # 参考帖子是否能产生选题
    reference_available_prompt = prompt.REFERENCE_NOTE_AVAILABLE_PROMPT.format(
        account_name=account_info.get("account_name"),
        account_description=account_info.get("description"), )
    reference_available_conversation_user_message = GenaiConversationMessage.one("user",
                                                                                 reference_available_prompt)

    system_prompt_count = google_genai.google_genai_client.models.count_tokens(model=model,contents=prompt.SYSTEM_INSTRUCTION_PROMPT)
    print("system_prompt_count ---", system_prompt_count)

    script_ans_conversation_model_message = google_genai.google_genai_output_images_and_text(
        reference_available_conversation_user_message,
        model=model,
        history_messages=history_messages,
        system_instruction_prompt=prompt.SYSTEM_INSTRUCTION_PROMPT)
    ans_content = script_ans_conversation_model_message.content[0].value

    return ans_content, [script_ans_conversation_model_message.usage_metadata]


def topic_generate_by_persona(topic_persona, reference_note_id):
    model = google_genai.MODEL_GEMINI_2_5_PRO_EXPT_0325
    history_messages = []
    reference_note = _get_note_detail_by_id(reference_note_id)
    # 参考帖子
    if reference_note.get("content_type") == "video" and reference_note.get("video", {}).get("video_url") is not None:
        reference_note_medias = [reference_note.get("video", {}).get("video_url")]
    else:
        reference_note_medias = [utils.oss_resize_image(i) for i in
                                 utils.remove_duplicates(reference_note.get("images"))]
    reference_note_prompt = prompt.REFERENCE_NOTE_PROVIDER_PROMPT.format(
        title=reference_note.get("title"),
        body_text=reference_note.get("body_text"),
        image_count=len(reference_note_medias)
    )
    reference_note_conversation_user_message = google_genai.GenaiConversationMessage.text_and_images(
        reference_note_prompt, reference_note_medias)
    history_messages.append(reference_note_conversation_user_message)

    topic_generate_by_persona_and_reference_note_prompt = prompt.TOPIC_GENERATE_BY_PERSONA_AND_REFERENCE_NOTE_PROMPT.format(
        persona_topic=topic_persona,
    )
    topic_generate_conversation_user_message = GenaiConversationMessage.one("user", topic_generate_by_persona_and_reference_note_prompt)


    script_ans_conversation_model_message = google_genai.google_genai_output_images_and_text(
        topic_generate_conversation_user_message,
        model=model,
        history_messages=history_messages,
        system_instruction_prompt=prompt.SYSTEM_INSTRUCTION_PROMPT)
    ans_content = script_ans_conversation_model_message.content[0].value

    return ans_content, [script_ans_conversation_model_message.usage_metadata]


if __name__ == "__main__":
    persona = """作为一名小红书人设构建专家，我对账号“胖胖胖胖总”进行了深度分析。以下是基于该账号历史帖子的客观总结：

### 账号核心信息
*   **账号名：** 胖胖胖胖总
*   **账号描述：** 🍬普本->猎头公司合伙人->资深HRD->集齐京沪深杭的职场博主；🍬曾在985大学授课|上过《三联生活周刊》|行业专家；📮合作：ppppzong@qq.com

### 账号内容分析

#### 1. 账号内容的品类
*   **核心品类：**
    *   **职场观察与解读：** 约占60%。涵盖大厂文化、职场PUA、裁员、内卷、升职路径、职业发展等。
    *   **社会热点深度剖析：** 约占30%。以时下热门“吃瓜”事件为载体，剖析其背后的社会阶层、人性、教育、婚恋等深层逻辑。
    *   **个人成长与认知升级：** 约占10%。通过对职场与社会现象的观察，提炼出关于个人成长、认知纠错和心态调整的建议。
*   **一句话总结账号的核心内容品类和创作方向：** 该账号以资深HRD的独特视角，通过对职场生态及社会热点事件的深度“吃瓜”式解读，揭示现象背后的深层逻辑与人性真相，旨在为读者提供客观的职场与人生洞察，并促进认知升级。

#### 2. 账号内容的主题
*   **核心主题：**
    *   **职场生存法则与挑战：** 聚焦大厂“饥饿游戏”模式、内卷、PUA、裁员潮、晋升困境、不健康的职场关系等。
    *   **社会阶层与特权现象：** 深入探讨“天龙人”的“保级焦虑”、资源倾斜、投胎技术、家族联姻、学历“注水”等特权现象。
    *   **婚恋关系中的权力与利益：** 关注婚姻中的PUA、财产转移、伴侣关系的异化以及“校园到婚纱”童话的破碎。
    *   **认知偏差与自我成长：** 讨论“洗脑包”的危害、绩优主义的反噬、自我认知的重要性以及如何重建健康心态。
*   **主题细分：**
    *   **职场：** 大厂的组织架构、绩效考核、领导风格、HR职能、员工心理健康（如焦虑、抑郁）、高压工作环境下的身体反应。
    *   **社会现象：** 名人/高知家庭的育儿方式、留学经历对人性的影响、财富对人品性的考验、社会分层与固化、不同职业/行业的就业前景。
    *   **人际关系：** 亲子关系中的PUA、伴侣关系中的不平等、社会交往中的“正常人”与“不正常人”区分。
*   **主题关联性：** 各主题之间存在高度关联。职场问题常被视为社会结构性问题的映射；婚恋、家庭、教育等话题也常与阶层、特权、个人成长等核心概念交叉。例如，大厂PUA技能被认为也体现在家庭关系中；高知家庭的育儿问题被视为其“保级焦虑”的体现。

#### 3. 账号内容的特点、风格、一致性
*   **特点：**
    *   **犀利批判：** 敢于直言不讳地指出问题，不回避敏感话题。
    *   **深度洞察：** 擅长从表面现象挖掘其深层逻辑和普遍规律。
    *   **幽默讽刺：** 运用大量网络表情包和调侃语气，将沉重话题轻松化，提升可读性。
    *   **务实清醒：** 强调回归现实，打破幻想，提供具有实践意义的建议。
    *   **冷静客观（表象）：** 尽管观点犀利，但表达方式多以转述、引用聊天记录为主，营造出一种“旁观者清”的客观感。
*   **风格：**
    *   **对话式呈现：** 大部分内容以微信聊天截图的形式呈现，模拟朋友间的私密讨论，增强真实感和代入感。
    *   **图文结合：** 搭配大量表情包和网络梗图，增加趣味性和情感表达。
    *   **视频化解读：** 部分内容制作成配有字幕和表情包的语音视频，便于多维度传播复杂信息。
    *   **口语化与专业性并存：** 语言风格轻松活泼，同时又不失专业术语和理论分析。
*   **一致性：** 账号在所有帖子中都保持了高度一致的特点和风格。
    *   **人设一致性：** 胖胖胖胖总作为“资深HRD”和“猎头公司合伙人”，其发言总能体现出对职场和人性深刻的理解，以及对社会“洗脑包”的警惕。其描述的“普本”背景与“专家”身份形成反差，增强了其观点的可信度。
    *   **表达方式一致性：** 几乎所有帖子都采用聊天截图作为主要内容载体，辅以简洁明了的文字总结和观点输出，表情包的使用也高度统一。
    *   **内容方向一致性：** 始终围绕职场痛点、社会热点中的不公现象，以及由此延伸出的个人成长和认知升级。

#### 4. 账号内容的痛点/需求/兴趣点
*   **痛点：**
    *   职场PUA和内卷带来的精神压力和身体健康问题。
    *   对社会阶层固化和特权现象的无力感。
    *   对传统婚恋观念和“成功”定义的反思与迷茫。
    *   信息茧房和认知偏差导致的生活困境。
*   **需求：**
    *   获得对复杂社会和职场现象的深度解读和真相。
    *   寻找共鸣，确认自身经历并非个例，从而缓解焦虑和孤独。
    *   学习如何识别和应对职场及生活中的PUA、不公平待遇。
    *   获取务实的职业发展和人生选择建议。
*   **兴趣点：**
    *   围绕知名大厂和公众人物的“吃瓜”事件。
    *   对社会底层逻辑和人性本质的探讨。
    *   反思和批判主流价值观。

#### 5. 账号内容的目标受众画像
*   **核心受众：** 25-40岁之间的职场人士，尤其是互联网大厂或高压行业从业者。对社会现实有一定认知，但可能在职场或个人生活中遇到困境。
*   **受众特点：**
    *   **职业背景：** 主要集中在互联网、金融、医疗等竞争激烈、压力较大的行业。
    *   **思维方式：** 具有一定的批判性思维，不盲从主流观点，渴望了解事物真相。
    *   **情感状态：** 可能存在职场焦虑、精神内耗、对未来感到迷茫或失望。
    *   **生活关注：** 除了职业发展，也关注个人成长、婚恋关系、家庭教育以及社会公平议题。
    *   **学习习惯：** 倾向于从案例和故事中学习，喜欢直接、不拖沓的观点输出。
*   **为什么这些受众会关注这个账号：** 该账号的深层分析和接地气的表达方式，能够精准触达这些受众的痛点，提供情绪价值（共鸣、解气）和认知价值（洞察、解决方案）。“胖胖胖胖总”的人设也增强了内容的可信度和吸引力。

#### 6. 账号内容的内容价值主张
*   **核心价值主张：** 击碎滤镜与“洗脑包”，提供职场与社会真相的深度解读，赋能个体以清醒的认知和实用的策略应对复杂现实。
*   **所属维度：**
    *   **信息获取：** 提供独家视角、深度解读、系统知识。
    *   **情感满足：** 获得共鸣、治愈、激励、认同感。
    *   **技能提升：** 提供应对职场PUA、调整认知的实用方法论。

#### 7. 账号内容的内容深度与广度
*   **深度：** **专业壁垒**。账号不仅仅停留在事件的表面，而是深入挖掘事件背后的社会结构、心理机制、权力博弈和人性弱点。例如，将职场PUA与家庭PUA、国际关系进行类比，将高知家庭的育儿焦虑与阶级固化相连接。这需要深厚的社会学、心理学和职场经验积累。
*   **广度：** **适度拓展**。账号以“职场观察与解读”和“社会热点深度剖析”为核心，并围绕这些核心主题向外延伸，如“婚恋关系”、“个人成长”、“教育”、“留学”等，但始终保持在社会人文和职业发展的范畴内，没有过度泛化。

#### 8. 账号内容的亮点
*   **独特的人设：** “胖胖胖胖总”的资深HRD/猎头/专家身份，结合其看似随意的“吃瓜”方式，形成一种反差萌与权威感的独特结合。
*   **“吃瓜”的深度与广度：** 将日常吃瓜事件提升到对社会规律、人性本质的探讨，具有极强的启发性。
*   **痛点共鸣：** 精准捕捉职场人普遍存在的焦虑、内耗、不公等痛点，提供情绪宣泄和理解的出口。
*   **形式创新：** 大量使用微信聊天截图作为内容主体，辅以画外音和表情包，增强了内容的真实感、亲切感和传播效率。
*   **反“洗脑包”：** 致力于揭露社会中的虚假繁荣和不切实际的“成功学”，帮助用户建立更客观清醒的认知。

#### 9. 账号内容的选题模式
*   **选题的灵感来源：**
    *   **社会热点事件/名人八卦：** 尤其是高关注度的、涉及特权阶层、大厂或婚恋纠纷的事件（如“大连王博文老师外甥故事”、“互联网大厂技术高管婚变”、“董天临/翟天临事件”、“大厂心理咨询师辞职”、“甜甜小姐代言风波”、“特朗普美股操作”）。
    *   **职场普遍现象：** 如职场PUA、大厂的“饥饿游戏”模式、内卷、裁员、职场中的“黑话”和不健康沟通方式。
    *   **个人成长与心理困惑：** 例如认知偏差、讨好型人格的形成、精神内耗、自我主体性的缺失。
    *   **粉丝互动与提问：** 评论区提问、私信咨询等，也成为重要的选题来源。
    *   **个人职业经验：** 作为资深HRD和猎头，对行业内部动态和人才趋势有敏锐的洞察。
*   **选题的创作方式：**
    *   **案例引入-深度解析模式：** 从一个具体的“瓜”或事件（如“大厂驻厂心理咨询师跑路”）开始，通过微信聊天记录的形式呈现，然后由博主进行深入的分析和点评，将个体事件上升为普遍现象或系统性问题。
    *   **反向观点-批判性解读模式：** 针对某个主流或普遍接受的观念（如“高知高认知”、“校园到婚纱是圆满剧本”），提出质疑并进行反向的、批判性的解读。
    *   **现象归纳-本质揭示模式：** 观察多个看似独立的现象（如“大厂各种PUA”，以及“家庭PUA”），归纳出其共同的底层逻辑或本质规律（如“权力博弈”、“控制欲”）。
*   **选题的亮点体现：**
    *   **信息差的弥补：** 提供普通人难以触及的内幕信息或专业视角。
    *   **情绪价值的满足：** 观点犀利，直指痛点，让读者感到被理解和共鸣。
    *   **认知升级的引导：** 每次分析都试图打破用户的固有认知，促使他们进行深度思考。
*   **历史优质选题（基于点赞数和收藏数）：**
    *   **帖子3: 怎么就…突然有了自我**
        *   **亮点：** 探讨了社会地位与育儿结果的非正相关性，以及高精专父母在孩子面前的两幅面孔。深刻揭示了“突然有了自我”背后可能存在的社会时钟、家庭 PUA 和“偶像包袱”等问题。
        *   **人设结合：** 胖胖胖胖总作为职场与个人成长专家，敏锐捕捉到社会热点中反映出的深层人性与家庭教育问题，其犀利但不失思辨的分析，符合其“击碎洗脑包”的人设。
        *   **创作方式：** 以大连王博文老师分享的故事为引子，通过聊天截图呈现讨论过程，逐步引入“社会时钟”、“自我主体性”等概念，由浅入深。
        *   **内容方向：** 个人成长、家庭教育、社会时钟、高知群体心态、认知偏差。
        *   **灵感来源：** 网络上关于大连王博文老师外甥的故事。
    *   **帖子15: 绷不住了，大厂驻厂心理咨询师跑路了…**
        *   **亮点：** 揭示了互联网大厂“饥饿游戏”式的生存模式，以及由此引发的员工精神状态问题，甚至连心理咨询师也难以自洽，强调这是结构性问题而非个体问题。
        *   **人设结合：** 胖胖胖胖总作为资深HRD，对大厂内部运作和职场PUA有着深刻理解，通过该事件验证了其“职场炼狱”的观点，进一步巩固了其专业且敢言的人设。
        *   **创作方式：** 从“大厂驻厂心理咨询师跑路”这一新闻事件切入，通过微信聊天截图展现朋友间的讨论，引申到大厂普遍存在的系统性压榨和精神污染问题。
        *   **内容方向：** 互联网大厂职场文化、员工心理健康、职场PUA、企业结构性问题。
        *   **灵感来源：** 某互联网大厂心理咨询师的自白。
    *   **帖子6: 不是…这也算立功了吧🐶**
        *   **亮点：** 巧妙地将“董天临”事件与“计划生育”联系，讽刺了某些特权阶层子女因父母的铺路而缺乏实际能力，最终导致“培养失败的联姻工具”被全国网友“扒”的现象，并将其视为是对社会公众的“立功”。
        *   **人设结合：** 胖胖胖胖总以其独特的幽默感和批判视角，对社会不公现象进行辛辣讽刺，体现了其“吃瓜”背后对真相的追求和对社会公平的关注。
        *   **创作方式：** 以“董天临”事件的后续进展为核心，通过聊天截图展现朋友间的讨论，从“家族联姻”、“草包小主”等角度进行解读，最终得出“天亮了”的结论。
        *   **内容方向：** 阶层固化、家族联姻、特权教育、社会舆论、公平正义。
        *   **灵感来源：** “董天临”事件的持续发酵和网友讨论。
    *   **帖子16: Dark Force的反面是“易被PUA”**
        *   **亮点：** 深入剖析了“鸡娃”现象背后的“要性”与“绩优主义”驱动，指出这种对“Dark Force”的追求可能使孩子更容易被PUA，并强调个人努力需考量能力边界与宏观环境。
        *   **人设结合：** 胖胖胖胖总结合心理学与职场经验，对当下育儿焦虑和社会成功学进行深层批判，展现了其对个体心理与社会文化关联的深刻洞察。
        *   **创作方式：** 从“鸡娃走火入魔”的现象切入，通过聊天截图和理论分析（均值回归、Dark Force），探讨了“要性”的双刃剑效应及其对个体心理健康的潜在风险。
        *   **内容方向：** 鸡娃、教育焦虑、绩优主义、职场PUA、原生家庭影响、个人成长。
        *   **灵感来源：** 育儿领域的热点话题及专家观点。
    *   **帖子11: 害，留子是一片大瓜田…🫣**
        *   **亮点：** 提供了对留学生群体的批判性视角，区分了“easy模式”下被过度保护和“一路水”的留学生，以及通过信息差和“钞能力”获得资源的现象，打破了对“留子”的单一滤镜。
        *   **人设结合：** 胖胖胖胖总作为社会观察者，敢于揭露教育资源不公和信息不对称的问题，其客观且不失犀利的分析符合其“击碎洗脑包”的人设。
        *   **创作方式：** 以“留子客观看待XX本科学校”的笔记为引子，通过聊天截图讨论留学生的不同成长路径，引申出信息差、金钱和背景在教育中的作用。
        *   **内容方向：** 留学生群体、教育公平、阶层差异、信息差、社会现实。
        *   **灵感来源：** 对留学生群体的观察和相关社会讨论。

#### 10. 账号创作新选题与人设匹配的判断条件
为了保持账号内容的一致性和专业性，创作新选题时需要判断新选题是否符合账号选题的人设。以下是判断新选题与人设匹配的方式和方法：

*   **核心匹配要素（必须满足）：**
    1.  **主题相关性：** 新选题必须紧密围绕“职场生态”、“社会热点（尤其是涉及权力、利益、人性、阶层等）”或“个人成长与认知升级”这三个核心品类。
    2.  **批判性与深度：** 选题不能是表面化的“吃瓜”，而应能深挖其背后的逻辑、系统性问题或人性弱点，体现“胖胖胖胖总”的深度洞察力。
    3.  **人设契合度：** 选题应能让“胖胖胖胖总”以资深HRD/猎头/专家身份，结合其独特的“普本”背景，提供具有专业性、清醒、甚至略带玩世不恭但务实的观点。
    4.  **形式延续性：** 优先考虑使用微信聊天截图作为主要内容呈现形式，辅以精炼的文字总结和博主的评论，或制作成配有字幕和表情包的语音视频。

*   **辅助匹配方式：**
    1.  **“洗脑包”识别：** 选题是否能揭示某种社会“洗脑包”或普遍存在的认知误区，并提供反思和纠正的视角。
    2.  **情绪共鸣与引导：** 选题能否触及目标受众的痛点，引发强烈共鸣，并引导读者从情绪宣泄走向理性思考和自我赋能。
    3.  **跨界关联性：** 尝试将看似不相关的职场/社会现象进行巧妙的类比和连接，展现博主的思维广度（例如将职场PUA与家庭PUA、国际关系相连）。
    4.  **反差与幽默：** 选题本身或其解读方式是否具有一定的反差感，能够通过幽默、讽刺或调侃的语气来化解沉重，提升传播力。

*   **不匹配的信号（应避免）：**
    *   纯粹的娱乐八卦，缺乏深度分析和普适价值。
    *   过于积极正向、脱离现实的“鸡汤”内容。
    *   与职场、社会议题、个人成长主题无关的生活分享。
    *   表达方式过于情绪化，缺乏冷静客观的分析。
    *   内容立场模糊，或与过往人设观点产生冲突。
"""
    # ans, usage = reference_note_available("56e02fa4cb35fb071599c959", "682d78b1000000002100468f")

    ans, usage = topic_generate_by_persona(persona, "683015460000000023012e14")
    print(usage)
    pass
