import json
from image_article_comprehension.aiddit.model import gemini, claude35
import image_article_comprehension.aiddit.utils as utils


def prompt_note_script(note):
    prompt = f'''
我会给出图片和以下小红书的帖子信息：
图片一共{len(utils.remove_duplicates(note.get("images", [])))}张，
第一张图片为封面图，其余的图片为图集；
我给出的每一个图片文件整体表示为一张图片；

帖子的标题如下：
"""
{note['title']}
"""

帖子正文如下：
"""
{note['body_text']}
"""

以上就是我提供的全部内容。

请你从内容创作的角度，根据我给出的内容，完成以下任务：
- 从内容创作的角度，对这条小红书的帖子进行创作脚本的拆解；
    - 拆解出来的创作脚本是能够完整的从脚本来还原创作复刻这条帖子的内容；
    - 拆解出来的创作脚本是对帖子内容的一个完整的解读；从【结构化】、【逻辑】的层面来进行拆解；
    - 视觉构建方式：包括但不限于以下部分，作为后续输出的引用：
        - 图片是否是实拍，还是非实拍？如果是非实拍，则详细描述图片；
        - 图片的视觉风格是什么？比如：Ins风、日系治愈、国潮复古、极简冷淡、手绘涂鸦、科技感、梦幻唯、胶片质感美等等；
        - 图片类型的分布：产品图/场景图/步骤图/对比图/表情包/文字信息	等等；
        - 图片中是否需要有添加了文字说明，如果有文字说明，则文字说明从哪里来，有什么特点？字体设计的原则（字体样式、字体大小等等维度去描述）；
        - 图片是否进行了拼接？如何拼接，拼接的方式等等；
        - 图片是否进行了二次加工？如何加工；
        - 从创作逻辑上总结：图片的构图的特点(dict)、构图详情(dict)；
        - 如果是视觉上的元素是有规则的组合，那么详细描述排版结构(dict)，以及排版逻辑(dict)；
        - 等等等其他跟视觉相关的特性，
        - 每一个特性的对应的value尽可能都输出为dict，这样你才能进行详细的描述；
    - 我会给你以下几个方面的提示可以帮助你去思考，并给我结果：
        - 整体规划
            - 指的是对这个帖子的创作，整体层面的规划；
            - 标题、正文、图片之间的关系如何？如何环环相扣从而产生一个好内容？
            - 内容侧重与图片还是文字？还是图文都重要？为什么？
            - 等等其他信息
        - 标题
            - 标题的吸引点如何构建？
            - 核心主题怎么体现？
            - 写作逻辑如何？
            - 等等其他信息
        - 正文
            - 正文的结构如何？
            - 写作技巧、手法、方式如何？
            - 内容的逻辑如何？
            - 等等其他信息；
        - 图片
            - 封面(dict)
                - 封面的选取逻辑（dict）: 为什么选取了这张图片作为封面？其背后的逻辑是什么？
                - 封面类型(dict)：比如 九宫格拼图、对比图、产品特写、真人场景、文字大字报等等类型的；
                - 视觉构建方式(dict)：关于`视觉构建方式`在上面已经进行解释说明，；
                - 其他重要信息（dict）
            - 图集(dict)
                - 图集中图片的数量规划（dict）
                - 图集的数量规划逻辑（dict）
                - 图片叙事逻辑（dict）：比如 问题→解决、Before-After、使用场景串联、产品360°展示、效果对比实验等等；
                - 图集之间的顺序怎么安排？排序的逻辑、依据是什么？
                - 视觉构建方式(dict)：关于`视觉构建方式`在上面已经进行解释说明，请注意从图集整体的角度来回答；
            - 封面和图集的关联关系(dict)
            - 封面和图集的通用信息(dict)
    - 请参考（包含但不限于）上述提示，output in JSON format with keys:
        - 整体规划(dict)
        - 标题(dict)
        - 正文(dict)
        - 图片(dict):
            - 封面(dict)
            - 图集(dict)
            - 封面和图集的关联关系(dict)
            - 封面和图集的通用信息(dict)
要求：
- 输出为中文；
- 请只描述客观事实，不要加入任何```主观```评价性语言；
- 请不要只进行概括性的描述
    - 比如：标题很吸引人，正文很有趣，图片很好看，构图简洁明了，这样是不允许的；
    - 要求进行详细的描述，比如构图特点具体是详细描述到怎么进行构图，详细描述到排版的逻辑是什么；
    - 禁止出现'氛围'、'和谐'、'视觉呈现','美学'等指代无不明确的用词
- 输出为JSON，不要有除了JSON格式之外的其他输出；
    '''.strip()

    print(prompt)
    # ans = claude35(prompt, image_list=note.get("images", []))
    ans = gemini(prompt, model="gemini-2.0-flash", images=note.get("images", []), temperature=0.4)
    return ans


def prompt_note_script_0125(note):
    prompt = f'''
我会给出图片和以下小红书的帖子信息：
帖子的标题如下：
"""
{note['title']}
"""

帖子正文如下：
"""
{note['body_text']}
"""

以上就是我提供的全部内容。

请你从内容创作的角度，根据我给出的内容，依次完成以下任务：
- 理解这条内容的核心主题和内容亮点；
- 基于核心主题和亮点，完成内容的创作脚本的拆解，因为脚本都是围绕内容核心注意和亮点进行创作：    
    - 拆解出来的创作脚本是能够完整的从脚本来还原创作复刻这条帖子的内容；
    - 拆解出来的创作脚本是对帖子内容的一个完整的解读；
    - 注意：我给出的所有图片中 第一张图片为封面图；其余的图片为图集；
    - 创作脚本一定是高度关联到内容的核心主题和亮点；
    - 深度思考标题、正文、图片是如何展现内容的核心主题和亮点；
    - 创作脚本包含以下部分：
        - 整体规划(dict)：指的是对这个帖子的创作，整体层面的规划
            - 核心主题(str)
            - 内容亮点(str)
            - 标题、正文、图片之间的关系如何？如何环环相扣从而产生一个好内容？
            - 内容侧重与图片还是文字？还是图文都重要？为什么？
            - 等等其他信息
        - 主体(dict)：
            - 提取帖子的主体：演绎该主题的主要人物/物品/动物等等；
            - 主体的详细描述：越详细越好，如果是人，则需要涵盖其性别、样貌等等；如果是动画，则需要涵盖具体的动物品种，样式等等；
            - 主体的特点：主体的特点是什么？
        - 标题(dict)
            - 标题的吸引点如何构建？
            - 核心主题怎么体现？
            - 写作逻辑如何？
            - 等等其他信息
        - 正文(dict)
            - 正文的结构如何？
            - 写作技巧、手法、方式如何？
            - 内容的逻辑如何？
            - 等等其他信息；
        - 图片(dict):
            - 封面(dict)：
                - 封面的展现形式(dict)：比如是单张图片还是多张图片拼接而成的；比如是否上面有文字等等；
                - 封面的选取逻辑(str)：为什么选取了这张图片作为封面？
                - 封面与主题的关系(str)：图片是如何表现内容主题或者亮点的？
            - 图集(List[dict])：
                - 图集的构建逻辑(dict): 从整理上来描述图集的构建逻辑，包含但不限于以下内容：
                    - 图集的数量规划(dict)：图集的数量是多少？为什么是这么多？
                    - 图集排序的规则(dict)：图集的内容是怎么构建的？为什么这么构建？
                - 图片列表(list[dict]): 图集中的每一张图片(dict)，包含但不限于以下内容：
                    - 图片序号(int) 
                    - 图片的呈现形式(dict)：图片是如何展现的？比如是单张图片还是多张图片拼接而成的；比如是否上面有文字等等；加工方式等等详细信息；
                    - 图片与主题的关系(str)：图片是如何表现内容主题或者亮点的？
            - 图片加工(dict | None)
            - 其他(dict | None)
要求：
- 输出创作脚本，结果为中文；
- 你的理解不要浮于表面，要求详细具体，应该从内容创作的角度去进行深度思考；
- 请只描述客观事实，不要加入任何'主观'评价性语言；
- 要求用词精准，不要使用模糊、不确定的词语；
- 语言表达上注意不要使用倒装句、长句、复杂句，尽量使用陈述句、简单句；
- 请只描述客观事实，不要加入任何```主观```评价性语言；
- 输出为JSON，不要有除了JSON格式之外的其他输出；
    '''.strip()

    print(prompt)
    ans = claude35(prompt, image_list=note.get("images", []))
    # ans = gemini(prompt, model="gemini-2.0-flash-exp", images=note.get("images", []))

    return ans


def prompt_script_summary(renshe_xuanti_unique, comprehension_note_list, renshe_xuanti_mode):
    script_list = []

    for note in comprehension_note_list:
        script_list.append(note.get("script", {}))

    prompt = f'''   
你是一名小红书的创作大师，拥有以下能力：
1. 对创作模式有着敏锐的洞察力，能够快速发现账号的创作模式；
2. 拥有丰富的创作经验；
3. 对创作模式有着独特的见解；能够从第一性原理出发，发现账号的脚本创作模式；
4. 拥有极高的内容总结、抽象能力；

以下是创作者的在该选题模式下的历史创作脚本：
"""
{json.dumps(script_list, ensure_ascii=False, indent=4)}
"""

请基于历史脚本，完成以下任务：
- 总结账号的创作脚本模式；
- 每个创作脚本模式有区分度，即该模式下的创作方式与其他模式下的创作方式有明显的区别；
- 创作脚本模式是需要有代表性的，如果历史选题中的该创作脚本模式样本比较小，则不适合提取创作脚本模式；
- 总结创作脚本的目的是为了让其他人能够通过该创作脚本模式，快速的了解 & 复刻账号的创作方式，请从目的出发，尽可能的详细；
- 挖掘该账号的创作脚本模式，账号的创作脚本模式应该是一个或多个，每一个创作脚本模式由以下结构组成：
    - 创作脚本模式名
        - 脚本模式名指的是该创作脚本模式的创作方式；
        - 模式应该'xxxx模式'的表达形式；
        - 模式名能够表达出来的是脚本创作的方式，不要有内容品类的描述，重点在表达怎么怎么创作的方式；
        - 从语义结构上能够对该脚本创作的方式能够有一个直观的认识；
    - 创作脚本模式的特点
        - 脚本模式的特点是该模式下的创作脚本的特点；
    - 创作脚本模式详情
        - 模式详情是指总结出来的脚本模式的具体内容；
        - 模式详情请尽可能的详细；包括但不限于：
            - 整体规划
            - 标题
            - 正文
            - 图片
                - 封面如何选取
                - 图片分段逻辑
                - 具体如何分段
                - 等等

要求如下：
- 请只描述客观事实，不要加入任何'主观'评价性语言；
- 要求用词精准，不要使用模糊、不确定的词语；    
- 结果尽可能详细，信息量丰富；
- 请直接输出JSON格式，不要有除了JSON格式之外的其他输出；
- direct output in JSONObject with keys:
    - modes (list[]) with each mode containing:
        - 创作脚本模式名(str)
        - 创作脚本模式的特点(dict)
        - 创作脚本模式详情(dict)
            - 整体规划(dict)
            - 主角(dict)
            - 标题(dict)
            - 正文(dict)
            - 图片(dict)
'''.strip()

    print(prompt)
    ans = claude35(prompt)
    return ans


def prompt_script_summary_0213(comprehension_note_list):
    script_list = []

    for note in comprehension_note_list:
        script_list.append(note.get("script", {}))

    prompt = f'''   
你是一名小红书的创作大师，拥有以下能力：
1. 对创作模式有着敏锐的洞察力，能够快速发现账号的创作模式；
2. 拥有丰富的创作经验；
3. 对创作模式有着独特的见解；能够从第一性原理出发，发现账号的脚本创作模式；
4. 拥有极高的内容总结、抽象能力；

以下是创作者的在该选题模式下的历史创作脚本：
"""
{json.dumps(script_list, ensure_ascii=False, indent=4)}
"""

请基于历史脚本，完成以下任务：
- 对小红书的每个模块（标题、正文、封面、图集）的创作进行模式上的总结；

- 每个模块由以下结构组成：
    - 通用信息
        - 通用信息是指在进行模块创作时，该模块的一些通用信息；
        - 并且标注哪些通用信息是必须，哪些是可选的；
        - 通用信息的结果明确具体，不要有模糊不清的信息，必须要有完整的信息量；
    - 创作模式
        - 模式的总结应该包含其主要的创作方式，以及该环节和其他模块之间的关系等信息；
            - 除了创作方式和模块关系外，你还需要适当补充其他信息；
            - 比如模式特点、模式适用场景等等；
            - 你提供的信息越丰富，越有助于其他人理解该模块的创作方式；
        - 每一种模式都有自己独特的创作方式，且该创作方式与其他模式有明显的区别；
        - 每一个模块的创作模式应该是一个或多个，每一个创作模式由以下结构组成：
            - 创作模式名:xxx模式（key） ： 创作模式详情（value）
        - 每一个模式要重点从怎么表达呈现内容主题的角度出发去总结其创作方式；
        - 每个模块必须包含一种模式；
        - 封面创作模式中的每一种模式的总结要求如下：
            - 封面是如何选取的？为什么选取这张图片作为封面？
            - 封面是独立生成的，还是由图集中的图片选取的、以及选取图片后的加工方式等等；
        - 图集创作模式中的每一种模式的总结要求如下：
            - 图集的数量规划(dict)：图集的数量是多少？为什么是这么多？
            - 图片之间的排序规则(dict)：图集的内容是怎么构建的？为什么这么构建？
            - 图片的创作方式(dict)
            - 与其他模块的关系(dict)

要求如下：
- 请只描述客观事实，不要加入任何'主观'评价性语言；
- 要求用词精准，不要使用模糊、不确定的词语；    
- 结果尽可能详细，信息量丰富；
- 请直接输出JSON格式，不要有除了JSON格式之外的其他输出；
- direct output in JSONObject with keys:
    - 标题
        - 通用信息(dict)
        - 创作模式(dict)
            - xx模式: dict
    - 正文(dict)
        - 通用信息(dict)
        - 创作模式(dict)
            - xx模式: dict
    - 封面(dict)
        - 通用信息(dict)
        - 创作模式(dict)
            - xx模式: dict
    - 图集(dict)
        - 通用信息(dict)
        - 创作模式(dict)
            - xx模式: dict
'''.strip()

    print(prompt)
    # reason, ans = deepseek_r3_stream(prompt)
    ans = gemini(prompt)
    print(ans)
    return ans


def prompt_note_script_logic(note):
    prompt = f'''
我会给出图片和以下小红书的帖子信息：
图片一共{len(utils.remove_duplicates(note.get("images", [])))}张，
第一张图片为封面图，其余的图片为图集；
我给出的每一个图片文件整体表示为一张图片；

帖子的标题如下：
"""
{note['title']}
"""

帖子正文如下：
"""
{note['body_text']}
"""

以上就是我提供的全部内容。

请你从内容创作的角度，根据我给出的内容，完成以下任务：
- 从内容创作的角度，对这条小红书的帖子进行创作脚本的拆解；
    - 拆解出来的创作脚本是能够完整的从脚本来还原创作复刻这条帖子的内容；
    - 拆解出来的创作脚本是对帖子内容的一个完整的解读；从【结构化】、【逻辑】的层面来进行拆解；
    - 我会给你以下几个方面的提示可以帮助你去思考，并给我结果：
        - 整体规划
            - 指的是对这个帖子的创作，整体层面的规划；
            - 标题、正文、图片之间的关系如何？如何环环相扣从而产生一个好内容？
            - 内容侧重与图片还是文字？还是图文都重要？为什么？
            - 等等其他信息
        - 标题
            - 标题的吸引点如何构建？
            - 核心主题怎么体现？
            - 写作逻辑如何？
            - 等等其他信息
        - 正文
            - 正文的结构如何？
            - 写作技巧、手法、方式如何？
            - 内容的逻辑如何？
            - 等等其他信息；
        - 图片
            - 封面(dict)
                - 封面的选取逻辑（dict）: 为什么选取了这张图片作为封面？其背后的逻辑是什么？
                - 其他重要信息（dict）
            - 图集(dict)
                - 图集中图片的数量规划（dict）
                - 图集的数量规划逻辑（dict）
                - 图集中图片的排列/顺序逻辑（dict）
                    - 图集之间的顺序怎么安排？排序的逻辑、依据是什么？
            - 封面和图集的关联关系(dict)
            - 封面和图集的通用信息(dict)
    - 请参考（包含但不限于）上述提示，output in JSON format with keys:
        - 整体规划(dict)
        - 标题(dict)
        - 正文(dict)
        - 图片(dict):
            - 封面(dict)
            - 图集(dict)
            - 封面和图集的关联关系(dict)
要求：
- 输出为中文；
- 请只描述客观事实，不要加入任何```主观```评价性语言；
- 请不要只进行概括性的描述
    - 比如：标题很吸引人，正文很有趣，图片很好看，构图简洁明了，这样是不允许的；
    - 禁止出现'氛围'、'和谐'、'视觉呈现','美学'等指代无不明确的用词
- 输出为JSON，不要有除了JSON格式之外的其他输出；
    '''.strip()

    print(prompt)
    # ans = claude35(prompt, image_list=note.get("images", []))
    ans = gemini(prompt, model="gemini-2.0-flash", images=note.get("images", []), temperature=0)
    return ans


def prompt_note_script_logic_vision(note):
    prompt = f'''
我会给出图片和以下小红书的帖子信息：
图片一共{len(utils.remove_duplicates(note.get("images", [])))}张，
第一张图片为封面图，其余的图片为图集；
我给出的每一个图片文件整体表示为一张图片；请注意一张图片中可能有多张图片组合拼接而成的情况；

帖子的标题如下：
"""
{note['title']}
"""

帖子正文如下：
"""
{note['body_text']}
"""

以上就是我提供的全部内容。

请你从内容创作的角度，根据我给出的内容，完成以下任务：
- 对每一张图片进行详细的描述，要求客观、详细、完整的描述；
- 从内容创作的角度，对这条小红书的帖子进行创作脚本视觉层面的拆解；
    - 视觉包括封面和图集；
    - 拆解出来的创作脚本是能够完整的从脚本来还原创作复刻这条帖子的内容；
    - 为了完整的表达内容的主题，从图片上的视觉构建方式进行拆解；
    - 视觉构建方式：包括但不限于以下部分，作为后续输出的引用：
        - 图片是否是实拍，还是非实拍？如果是非实拍，则详细描述图片；
        - 图片的视觉风格是什么；
        - 图片中是否需要有添加了文字说明？
            - 如果有文字说明，则文字说明从哪里来，有什么特点？
        - 总结图集中的多张图片，是否存在模版的形式，如果有模版的形式，详细描述图片模版(dict);
            - 模版包括了图片的排版、文字的排版、图片和文字的关系等等；
            - 图片是否进行了拼接？如何拼接，拼接的方式等等；
            - 文字和图片的关系是怎么样的？文字和图片的排版逻辑是什么？
            - 构图特点是什么？比如：对比图、九宫格拼图、产品特写、真人场景、文字大字报等
            - 模板的内容包括但不限于以上我给的内容；请从实际的图片中描述、总结模版
        - 图片是否进行了二次加工？如何加工；
        - 等等等其他跟视觉相关的特性，
        - 每一个特性的对应的value尽可能都输出为dict，这样你才能进行详细的描述；
    - 我会给你以下几个方面的提示可以帮助你去思考，并给我结果：
        - 视觉特点(dict)
        - 封面和图集的关联关系(dict)
        - 封面和图集的通用信息(dict)
        - 封面(dict)
            - 视觉构建方式(dict)：关于`视觉构建方式`在上面已经进行解释说明，；
            - 其他重要信息（dict）
        - 图集(dict)
            - 视觉构建方式(dict)：关于`视觉构建方式`在上面已经进行解释说明，请注意从图集整体的角度来回答；
            - 其他重要信息（dict）

    - 请参考（包含但不限于）上述提示，output in JSON format with keys:
        - 图片描述(list[str])：
        - 视觉特点(dict)
        - 封面和图集的关联关系(dict)
        - 封面和图集的通用信息(dict):
        - 封面(dict)
        - 图集(dict)
要求：
- 输出为中文；
- 请只描述客观事实，不要加入任何```主观```评价性语言；
- 请不要只进行概括性的描述
    - 比如：标题很吸引人，正文很有趣，图片很好看，构图简洁明了，这样是不允许的；
    - 要求进行详细的描述，比如构图特点具体是详细描述到怎么进行构图，详细描述到排版的逻辑是什么；
    - 禁止出现'氛围'、'和谐'、'视觉呈现','美学'等指代无不明确的用词
- 输出为JSON，不要有除了JSON格式之外的其他输出；
    '''.strip()

    print(prompt)
    # ans = claude35(prompt, image_list=note.get("images", []))
    ans = gemini(prompt, model="gemini-2.0-flash", images=note.get("images", []), temperature=0)
    return ans


def prompt_note_script_0220(note):
    prompt = f'''
我会给出图片和以下小红书的帖子信息：
图片一共{len(utils.remove_duplicates(note.get("images", [])))}张，
第一张图片为封面图，其余的图片为图集；
我给出的每一个图片文件整体表示为一张图片；

帖子的标题如下：
"""
{note['title']}
"""

帖子正文如下：
"""
{note['body_text']}
"""

以上就是我提供的全部内容。

请你从内容创作的角度，根据我给出的内容，完成以下任务：
- 从内容创作的角度，对这条小红书的帖子进行创作脚本的拆解；
    - 拆解出来的创作脚本是能够完整的从脚本来还原创作复刻这条帖子的内容；
    - 拆解出来的创作脚本是对帖子内容的一个完整的解读；从【结构化】、【逻辑】的层面来进行拆解；
    - 创作脚本是类似于电影的剧本，但是我现在要对一个小红书的图文帖子进行创作脚本的拆解；
    - 我会给你以下几个方面的提示可以帮助你去思考，并给我结果：
        - 整体规划
            - 指的是对这个帖子的创作，整体层面的规划；
            - 标题、正文、图片之间的关系如何？如何环环相扣从而产生一个好内容？
            - 内容侧重与图片还是文字？还是图文都重要？为什么？
            - 为了表达内容的主题，整体规划是怎么样的？
            - 等等其他信息
        - 标题
            - 标题的吸引点如何构建？
            - 核心主题怎么体现？
            - 写作逻辑如何？
            - 等等其他信息
        - 正文
            - 正文的结构如何？
            - 写作技巧、手法、方式如何？
            - 内容的逻辑如何？
            - 等等其他信息；
        - 图片
            - 封面(dict)
                - 封面的选取逻辑（dict）: 为什么选取了这张图片作为封面？其背后的逻辑是什么？
                - 封面和内容主题的关系(dict)：图片是如何表现内容主题或者亮点的？
                - 其他重要信息（dict）
            - 图集(dict)
                - 图集中图片的数量规划（dict）
                - 图集的数量规划逻辑（dict）
                - 图集和内容主题的关系(dict)：图片是如何表现内容主题或者亮点的？
                - 图集中图片的排列/顺序逻辑（dict）
                    - 图集之间的顺序怎么安排？排序的逻辑、依据是什么？
            - 封面和图集的关联关系(dict)
            - 封面和图集的通用信息(dict)
    - 请参考（包含但不限于）上述提示，output in JSON format with keys:
        - 整体规划(dict)
        - 标题(dict)
        - 正文(dict)
        - 图片(dict):
            - 封面(dict)
            - 图集(dict)
            - 封面和图集的关联关系(dict)
要求：
- 输出为中文；
- 请只描述客观事实，不要加入任何```主观```评价性语言；
- 请不要只进行概括性的描述
    - 比如：标题很吸引人，正文很有趣，图片很好看，构图简洁明了，这样是不允许的；
    - 禁止出现'氛围'、'和谐'、'视觉呈现','美学'等指代无不明确的用词
- 输出为JSON，不要有除了JSON格式之外的其他输出；
    '''.strip()

    print(prompt)
    ans = claude35(prompt, image_list=note.get("images", []))
    # ans = gemini(prompt, model="gemini-2.0-flash", images=note.get("images", []), temperature=0)
    return ans

